@@ -34,9 +34,11 @@
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <glade/glade.h>
+#include <glade/glade-build.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomecanvas/gnome-canvas-pixbuf.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libgnome/libgnometypebuiltins.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -85,6 +87,17 @@ static const struct poptOption options[] = {
 	{ NULL } 
 };
 
+GtkWidget *
+get_widget (const char *name, const char *loc)
+{
+	GtkWidget *w;
+	w = glade_xml_get_widget (druid_data.xml, name);
+	if (!w) {
+		g_warning (_("Could not find widget named %s at %s"), name, loc);
+	}
+	return w;
+}
+
 void
 buddy_set_text_widget (GtkWidget *w, const char *s)
 {
@@ -174,38 +187,6 @@ update_crash_type (GtkWidget *w, gpointer data)
 	return FALSE;
 }
 
-static gboolean
-update_submit_type (GtkWidget *w, gpointer data)
-{
-	SubmitType new_type = GPOINTER_TO_INT (data);
-	GtkWidget *table, *box;
-	druid_data.submit_type = new_type;
-
-	table = GET_WIDGET ("email-to-table");
-	box = GET_WIDGET ("email-file-gnome-entry");
-	
-	switch (new_type) {
-	case SUBMIT_REPORT:
-		gtk_widget_hide (box);
-		gtk_widget_show (table);
-		break;
-	case SUBMIT_FILE:
-		gtk_widget_hide (table);
-		gtk_widget_show (box);
-		break;
-	case SUBMIT_TO_SELF:
-		gtk_widget_hide (table);
-		gtk_widget_hide (box);
-		break;
-	default:
-		break;
-	}
-
-	gtk_widget_queue_resize (GET_WIDGET ("email-vbox"));
-
-	return FALSE;
-}
-
 void
 on_gdb_go_clicked (GtkWidget *w, gpointer data)
 {
@@ -246,8 +227,23 @@ on_gdb_stop_clicked (GtkWidget *button, gpointer data)
 	gtk_widget_destroy (w);
 }
 
+static void
+on_list_button_press_event (GtkWidget *w, GdkEventButton *button, gpointer data)
+{
+	GtkTreeSelection *selection;
+
+	if (button->type != GDK_2BUTTON_PRESS)
+		return;
+	
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (w));
+	if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
+		return;
+	
+	on_druid_next_clicked (NULL, NULL);
+}
+
 void
-stop_progress ()
+stop_progress (void)
 {
 	if (!druid_data.progress_timeout)
 		return;
@@ -274,117 +270,147 @@ stock_pixmap_buddy (gchar *w, char *n, char *a, int b, int c)
 	return gtk_image_new_from_stock (n, GTK_ICON_SIZE_SMALL_TOOLBAR);
 }
 
+/* this is pretty fragile */
 void
-title_configure_size (GtkWidget *w, GtkAllocation *alloc, gpointer data)
+on_email_group_toggled (GtkWidget *w, gpointer data)
 {
-	gnome_canvas_set_scroll_region (GNOME_CANVAS (w), 0.0, 0.0,
-					alloc->width, alloc->height);
-	gnome_canvas_item_set (druid_data.title_box,
-			       "x1", 0.0,
-			       "y1", 0.0,
-			       "x2", (gfloat)alloc->width,
-			       "y2", (gfloat)alloc->height,
-			       "width_units", 1.0, NULL);
+	const char *widgets[] = { "email-mailer-radio", "email-sendmail-radio", "email-file-radio" };
+	const char *label = GTK_STOCK_GO_FORWARD;
 
-	gnome_canvas_item_set (druid_data.banner,
-			       "x", 15.0,
-			       "y", 24.0,
-			       "anchor", GTK_ANCHOR_WEST, NULL);
+	/* this utilizes the pigeon hole principle */
+	for (druid_data.submit_type = SUBMIT_GNOME_MAILER;
+	     druid_data.submit_type < SUBMIT_FILE;
+	     druid_data.submit_type++)
+		if (GTK_TOGGLE_BUTTON (GET_WIDGET (widgets[druid_data.submit_type]))->active)
+			break;
 
-	gnome_canvas_item_set (druid_data.logo,
-			       "x", (gfloat)(alloc->width - 48),
-			       "y", -2.0,
-			       NULL);
+
+	if (druid_data.submit_type == SUBMIT_FILE) {
+		gtk_widget_hide (GET_WIDGET ("mail-notebook"));
+		gtk_widget_hide (GET_WIDGET ("email-to-table"));
+		gtk_widget_show (GET_WIDGET ("email-save-in-box"));
+	} else {
+		GtkWidget *nb = GET_WIDGET ("mail-notebook");
+
+		gtk_widget_show (nb);
+		gtk_widget_show (GET_WIDGET ("email-to-table"));
+		gtk_widget_hide (GET_WIDGET ("email-save-in-box"));
+
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
+					       druid_data.submit_type);
+	}
+
+	switch (druid_data.state) {
+	case STATE_EMAIL_CONFIG:
+		if (druid_data.submit_type == SUBMIT_GNOME_MAILER)
+			label = _("_Start Mailer");
+		break;
+	case STATE_EMAIL:
+		if (druid_data.submit_type == SUBMIT_SENDMAIL)
+			label = _("_Send Report");
+		else
+			label = GTK_STOCK_SAVE;
+		break;
+	default:
+		break;
+	}
+
+	g_object_set (GET_WIDGET ("druid-next"),
+		      "label", label,
+		      NULL);
 }
 
 void
-side_configure_size (GtkWidget *w, GtkAllocation *alloc, gpointer data)
+queue_download_restart (GtkWidget *w, gpointer data)
 {
-	gnome_canvas_set_scroll_region (GNOME_CANVAS (w), 0.0, 0.0,
-					alloc->width, alloc->height);
-	gnome_canvas_item_set (druid_data.side_box,
-			       "x1", 0.0,
-			       "y1", 0.0,
-			       "x2", (gfloat)alloc->width,
-			       "y2", (gfloat)alloc->height,
-			       "width_units", 1.0, NULL);
+	if (druid_data.dl_timeout)
+		gtk_timeout_remove (druid_data.dl_timeout);
 
-	gnome_canvas_item_set (druid_data.side_image,
-			       "x", 0.0,
-			       "y", (gfloat)(alloc->height - 179),
-			       "width", 68.0,
-			       "height", 179.0, NULL);
+	druid_data.dl_timeout = gtk_timeout_add (500, (GtkFunction)start_bugzilla_download, NULL);
+}
+
+void
+on_proxy_settings_clicked (GtkWidget *w, gpointer data)
+{
+	GtkWindow *win = GTK_WINDOW (GET_WIDGET ("proxy-window"));
+	gtk_window_set_transient_for (win,
+				      GTK_WINDOW (GET_WIDGET ("druid-window")));
+	gtk_window_present (win);
+}
+
+void
+on_progress_start_clicked (GtkWidget *wid, gpointer data)
+{
+	GtkTreeView *w;
+	GtkListStore *store;
+
+	w = GTK_TREE_VIEW (GET_WIDGET ("product-list"));
+
+	g_object_get (G_OBJECT (w), "model", &store, NULL);
+
+	gtk_list_store_clear (store);
+	druid_data.product = NULL;
+
+	start_bugzilla_download ();
+}
+
+void
+on_progress_stop_clicked (GtkWidget *w, gpointer data)
+{
+	end_bugzilla_download (END_BUGZILLA_CANCEL);
+	load_bugzilla_xml ();
+}
+
+void
+on_debugging_options_button_clicked (GtkWidget *w, gpointer null)
+{
+	GtkWidget *nb;
+	int page;
+
+	nb = GET_WIDGET ("gdb-notebook");
+
+	page = gtk_notebook_get_current_page (GTK_NOTEBOOK (nb)) ? 0 : 1;
+	
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (nb), page);
+	gtk_button_set_label (GTK_BUTTON (w),
+			      page
+			      ? _("Hide Debugging Options")
+			      : _("Show Debugging Options"));
+}
+
+void
+on_email_default_radio_toggled (GtkWidget *w, gpointer data)
+{
+	static const char *widgets[] = {
+		"email-command-label",
+		"email-command-entry",
+		"email-terminal-toggle",
+		"email-remote-toggle",
+		NULL
+	};
+	const char **s;
+	gboolean custom = GTK_TOGGLE_BUTTON (GET_WIDGET ("email-custom-radio"))->active;
+
+	gtk_widget_set_sensitive (GET_WIDGET ("email-default-combo"), !custom);
+
+	for (s = widgets; *s; s++) 
+		gtk_widget_set_sensitive (GET_WIDGET (*s), custom);
 }
 
 static void
-init_canvi (void)
+build_custom_mailers (gpointer key, gpointer value, gpointer data)
 {
-	GdkPixbuf *pb;
-	GnomeCanvasItem *root;
-	GnomeCanvas *canvas;
-	PangoFontDescription *pfd;
+	GList **list = data;
+	*list = g_list_append (*list, key);
+}
 
-	canvas = GNOME_CANVAS (GET_WIDGET ("title-canvas"));
-	root = GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (canvas)));
+static void
+fixup_notebook (const char *name)
+{
+	GtkNotebook *nb = GTK_NOTEBOOK (GET_WIDGET (name));
 
-	druid_data.title_box = 
-		gnome_canvas_item_new (GNOME_CANVAS_GROUP (root),
-				       gnome_canvas_rect_get_type (),
-				       "fill_color", "black",
-				       "outline_color", "black", NULL);
-
-	pfd = pango_font_description_from_string ("Helvetica Bold 18");
-	druid_data.banner =
-		gnome_canvas_item_new (GNOME_CANVAS_GROUP (root),
-				       gnome_canvas_text_get_type (),
-				       "fill_color", "white",
-				       "font_desc", pfd,
-				       "anchor", GTK_ANCHOR_WEST,
-				       NULL);
-	pango_font_description_free (pfd);
-
-	pb = gdk_pixbuf_new_from_file (BUDDY_DATADIR "/bug-core.png", NULL);
-	druid_data.logo =
-		gnome_canvas_item_new (GNOME_CANVAS_GROUP (root),
-				       GNOME_TYPE_CANVAS_PIXBUF,
-				       "pixbuf", pb, NULL);
-
-
-	/*******************************************************/
-	canvas = GNOME_CANVAS (GET_WIDGET ("side-canvas"));
-	root = GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (canvas)));
-	druid_data.side_box =
-		gnome_canvas_item_new (GNOME_CANVAS_GROUP (root),
-				       gnome_canvas_rect_get_type (),
-				       "fill_color", "black",
-				       "outline_color", "black", NULL);
-
-	pb = gdk_pixbuf_new_from_file (BUDDY_DATADIR "/bug-flower.png", NULL);
-
-	druid_data.side_image =
-		gnome_canvas_item_new (GNOME_CANVAS_GROUP (root),
-				       GNOME_TYPE_CANVAS_PIXBUF,
-				       "pixbuf", pb, NULL);
-
-
-	/*******************************************************/
-	canvas = GNOME_CANVAS (GET_WIDGET ("gdb-canvas"));
-	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), -12.0, -12.0, 12.0, 12.0);
-	root = GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (canvas)));
-	
-	pb = gdk_pixbuf_new_from_file (BUDDY_DATADIR "/bug-buddy.png", NULL);
-	druid_data.throbber_pb = gdk_pixbuf_scale_simple (pb, 24, 24, GDK_INTERP_BILINEAR);
-	g_object_ref (pb);
-
-	druid_data.throbber =
-		gnome_canvas_item_new (GNOME_CANVAS_GROUP (root),
-				       GNOME_TYPE_CANVAS_PIXBUF,
-				       "pixbuf", druid_data.throbber_pb,
-				       "x", -6.0,
-				       "y", -6.0,
-				       "height", 24.0,
-				       "width", 24.0,
-				       NULL);
+	gtk_notebook_set_show_border (GTK_NOTEBOOK (nb), FALSE);
+	gtk_notebook_set_show_tabs   (GTK_NOTEBOOK (nb), FALSE);
 }
 
 /* there should be no setting of default values here, I think */
@@ -392,18 +418,19 @@ static void
 init_ui (void)
 {
 	GtkWidget *w, *m;
-	gchar *s;
+	char *s;
+	const char **sp;
 	int i;
-
+	const char *nbs[] = { "druid-notebook", "gdb-notebook", "mail-notebook", NULL };
+	
 	glade_xml_signal_autoconnect (druid_data.xml);
 
 	load_config ();
 
-	init_canvi ();
+	/* gtk_widget_set_default_direction (GTK_TEXT_DIR_RTL); */
 
-	w = GET_WIDGET ("druid-notebook");
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (w), FALSE);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (w), FALSE);
+	for (sp = nbs; *sp; sp++)
+		fixup_notebook (*sp);
 
 	druid_data.gnome_version = g_getenv ("BUG_BUDDY_GNOME_VERSION");
 
@@ -437,6 +464,27 @@ init_ui (void)
 		druid_data.crash_type = CRASH_CORE;
 	}
 
+	{
+		char *message = g_strconcat (_("Welcome to Bug Buddy, a bug reporting tool for GNOME.\n"
+					       "It will step you through the process of submitting a bug report."),
+					     druid_data.crash_type == CRASH_DIALOG
+					     ? _("\n\nYou are seeing this because another application has crashed.")
+					     : "",
+					     druid_data.crash_type != CRASH_NONE
+					     ? _("\n\nPlease wait a few moments while Bug Buddy obtains debugging information "
+						 "from the applications.  This allows Bug Buddy to provide a more useful "
+						 "bug report.")
+					     : "",
+					     NULL);
+		g_object_set (GET_WIDGET ("gdb-label"),
+			      "label", message,
+			      "selectable", TRUE,
+			      NULL);
+		g_free (message);
+	}
+
+	if (druid_data.crash_type != CRASH_NONE) 
+
 	/* package version */
 	buddy_set_text ("the-version-entry", popt_data.package_ver);
 
@@ -455,21 +503,6 @@ init_ui (void)
 	gtk_option_menu_set_history (GTK_OPTION_MENU (w), druid_data.crash_type);
 	update_crash_type (NULL, GINT_TO_POINTER (druid_data.crash_type));
 
-	/* init more ex-radio buttons */
-	m = gtk_menu_new ();
-	for (i = 0; submit_type[i]; i++) {
-		w = gtk_menu_item_new_with_label (_(submit_type[i]));
-		g_signal_connect (G_OBJECT (w), "activate",
-				  G_CALLBACK (update_submit_type),
-				  GINT_TO_POINTER (i));
-		gtk_widget_show (w);
-		gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
-	}
-	w = GET_WIDGET ("email-option");
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (w), m);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (w), druid_data.submit_type);
-	update_submit_type (NULL, GINT_TO_POINTER (druid_data.submit_type));
-
 	gnome_window_icon_set_from_default (GTK_WINDOW (GET_WIDGET ("druid-window")));
 
 	g_object_set (G_OBJECT (GET_WIDGET ("druid-about")),
@@ -494,17 +527,97 @@ init_ui (void)
 			bbox, GET_WIDGET ("druid-about"), TRUE);
 	}
 		
+	gtk_misc_set_alignment (GTK_MISC (GET_WIDGET ("druid-logo")),
+				1.0, 0.5);
+
+	buddy_set_text ("email-default-entry", _(druid_data.mailer->name));
+
+	{
+		GList *mailers = NULL;
+		g_hash_table_foreach (druid_data.mailer_hash, build_custom_mailers, &mailers);
+		gtk_combo_set_popdown_strings (GTK_COMBO (GET_WIDGET ("email-default-combo")),
+					       mailers);
+		g_list_free (mailers);
+	}
+
+	w = GET_WIDGET ("druid-next");
+	gtk_widget_set_direction (GTK_BIN (GTK_BIN (w)->child)->child,
+				  gtk_widget_get_direction (w) == GTK_TEXT_DIR_RTL
+				  ? GTK_TEXT_DIR_LTR
+				  : GTK_TEXT_DIR_RTL);
+
+	g_signal_connect (GET_WIDGET ("product-list"), "event-after",
+			  G_CALLBACK (on_list_button_press_event),
+			  NULL);
+
+	g_signal_connect (GET_WIDGET ("component-list"), "event-after",
+			  G_CALLBACK (on_list_button_press_event),
+			  NULL);
+
+	{
+		const char **w, *windows[] = { "druid-window", "proxy-window", "progress-window", NULL };
+		for (w = windows; *w; w++)
+			g_signal_connect (GET_WIDGET (*w), "delete-event", 
+					  G_CALLBACK (gtk_widget_hide_on_delete),
+					  NULL);
+	}
+
+	gtk_dialog_set_has_separator (GTK_DIALOG (GET_WIDGET ("proxy-window")), FALSE);
+
+	buddy_set_text ("desc-text",
+			"Description of Problem:\n\n\n"
+			"Steps to reproduce the problem:\n"
+			"1. \n"
+			"2. \n"
+			"3. \n\n"
+			"Actual Results:\n\n\n"
+			"Expected Results:\n\n\n"
+			"How often does this happen?\n\n\n"
+			"Additional Information:\n");
 
-#if 0
 	/* set the cursor at the beginning of the second line */
 	{
 		GtkTextBuffer *buffy;
 		GtkTextIter iter;
 
 		buffy = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GET_WIDGET ("desc-text")));
-		gtk_text_buffer_get_iter_at_line (buffy, &iter, 2);
+		gtk_text_buffer_get_iter_at_line (buffy, &iter, 1);
+		gtk_text_buffer_place_cursor (buffy, &iter);
 	}
-#endif
+}
+
+GtkWidget *
+make_image (char *widget_name, char *icon, char *s2, int stock, int i2)
+{
+	GtkWidget *w = NULL;
+
+	if (stock) {
+		GtkIconSize size = GTK_ICON_SIZE_BUTTON;
+
+		if (s2)
+			size = glade_enum_from_string (GTK_TYPE_ICON_SIZE, s2);
+
+		w = gtk_image_new_from_stock (icon, size);
+	} else {
+		GnomeFileDomain domain = GNOME_FILE_DOMAIN_APP_PIXMAP;
+		char *filename;
+		
+		if (s2)
+			domain = glade_enum_from_string (GNOME_TYPE_FILE_DOMAIN, s2);
+
+		filename = gnome_program_locate_file (NULL, domain, icon, TRUE, NULL);
+
+		if (filename)
+			w = gtk_image_new_from_file (filename);
+		else
+			g_warning (_("Could not find pixmap: %s (%d)\n"), icon, domain);
+		g_free (filename);
+	}
+
+	if (w)
+		gtk_widget_show (w);
+
+	return w;
 }
 
 gint
@@ -515,39 +628,49 @@ delete_me (GtkWidget *w, GdkEventAny *evt, gpointer data)
 	return FALSE;
 }
 
+void
+on_druid_window_style_set (GtkWidget *widget, GtkStyle *old_style, gpointer data)
+{
+	gtk_widget_modify_fg (GET_WIDGET ("druid-label"), GTK_STATE_NORMAL, &widget->style->fg[GTK_STATE_SELECTED]);
+	gtk_widget_modify_bg (GET_WIDGET ("druid-background"), GTK_STATE_NORMAL, &widget->style->bg[GTK_STATE_SELECTED]);
+}
+
 int
 main (int argc, char *argv[])
 {
 	GtkWidget *w;
 	char *s;
-
+		
 	memset (&druid_data, 0, sizeof (druid_data));
 	memset (&popt_data,  0, sizeof (popt_data));
 
 	druid_data.crash_type = CRASH_NONE;
 	druid_data.state = -1;
 
-	srand (time (NULL));
-
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
 
 	gnome_program_init (PACKAGE, VERSION,
 			    LIBGNOMEUI_MODULE,
 			    argc, argv,
 			    GNOME_PARAM_POPT_TABLE, options,
-			    GNOME_PARAM_POPT_FLAGS, 0,
+			    GNOME_PARAM_APP_DATADIR, REAL_DATADIR,
 			    NULL);
 
-	gnome_window_icon_set_default_from_file (BUDDY_ICONDIR"/bug-buddy.png");
+	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, 
+				       "bug-buddy.png", TRUE, NULL);
+	
+	if (s)
+		gnome_window_icon_set_default_from_file (s);
+	g_free (s);
 
-#if 0
-	s = "bug-buddy.glade2";
-	if (!g_file_tests (s))
-#endif
-		s = BUDDY_DATADIR "/bug-buddy.glade2";
-
-	druid_data.xml = glade_xml_new (s, NULL, GETTEXT_PACKAGE);
+	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
+				       "bug-buddy/bug-buddy.glade2", TRUE, NULL);
+	if (s)
+		druid_data.xml = glade_xml_new (s, NULL, GETTEXT_PACKAGE);
+	else
+		druid_data.xml = NULL;
 
 	if (!druid_data.xml) {
 		w = gtk_message_dialog_new (NULL,
@@ -564,23 +687,22 @@ main (int argc, char *argv[])
 		gtk_widget_destroy (w);
 		return 0;
 	}
+	g_free (s);
+
+	druid_set_state (STATE_GDB);
 
 	init_ui ();
 
 	gtk_widget_show (GET_WIDGET ("druid-window"));
 
-	if (getenv ("BUG_ME_HARDER"))
+	if (getenv ("BUG_ME_HARDER")) {
 		gtk_widget_show (GET_WIDGET ("progress-window"));
+		gtk_widget_show (GET_WIDGET ("proxy-window"));
+	}
 	
-	druid_set_state (STATE_INTRO);	
-
-	if (druid_data.already_run && 
-	    gtk_toggle_button_get_active (
-		    GTK_TOGGLE_BUTTON (GET_WIDGET ("intro-skip-toggle"))))
-		on_druid_next_clicked (NULL, NULL);
-
 	load_bugzillas ();
-
+	start_bugzilla_download ();
+	start_gdb ();
 
 	gtk_main ();
 
